#! perl
# Copyright (C) 2007, The Perl Foundation.
# $Id: auto_socklen_t-02.t 29323 2008-07-12 03:12:21Z jkeenan $
# auto_socklen_t-02.t

use strict;
use warnings;
use Test::More tests =>  15;
use Carp;
use lib qw( lib t/configure/testlib );
use_ok('config::init::defaults');
use_ok('config::auto::socklen_t');
use Parrot::Configure;
use Parrot::Configure::Options qw( process_options );
use Parrot::Configure::Test qw( test_step_thru_runstep);

my $args = process_options(
    {
        argv => [ ],
        mode => q{configure},
    }
);

my $conf = Parrot::Configure->new;

test_step_thru_runstep( $conf, q{init::defaults}, $args );

my $pkg = q{auto::socklen_t};

$conf->add_steps($pkg);
$conf->options->set( %{$args} );

my ( $task, $step_name, $step);
$task        = $conf->steps->[-1];
$step_name   = $task->step;

$step = $step_name->new();
ok( defined $step, "$step_name constructor returned defined value" );
isa_ok( $step, $step_name );


#my $ret = $step->runstep($conf);
#ok( $ret, "$step_name runstep() returned true value" );
#ok(defined($step->result()), "A result has been defiined");
my $d_socklen_t;

$d_socklen_t = q{alpha};
ok($step->_evaluate_socklen_t($conf, $d_socklen_t),
    "_evaluate_socklen_t() completed satisfactorily");
is($step->result(), q{yes}, "Got expected result");
is($conf->data->get('has_socklen_t'), 1,
    "has_socklen_t set as expected");

$d_socklen_t = undef;
ok($step->_evaluate_socklen_t($conf, $d_socklen_t),
    "_evaluate_socklen_t() completed satisfactorily");
is($step->result(), q{no}, "Got expected result");
is($conf->data->get('has_socklen_t'), 0,
    "has_socklen_t set as expected");


pass("Completed all tests in $0");

################### DOCUMENTATION ###################

=head1 NAME

auto_socklen_t-02.t - test config::auto::socklen_t

=head1 SYNOPSIS

    % prove t/steps/auto_socklen_t-02.t

=head1 DESCRIPTION

The files in this directory test functionality used by F<Configure.pl>.

The tests in this file test subroutines exported by config::auto::socklen_t.

=head1 AUTHOR

James E Keenan

=head1 SEE ALSO

config::auto::socklen_t, F<Configure.pl>.

=cut

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 100
# End:
# vim: expandtab shiftwidth=4:
